using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace MessageBoxGenerator
{
    public partial class FormMessage : Form
    {
        public FormMessage()
        {
            InitializeComponent();
        }

        public DialogResult Show(
            string message,
            string title,
            int buttonCount,
            Button button1,
            Button button2,
            Button button3,
            string iconFileName
            )
        {

            if (buttonCount > 0)
            {
                SetButton(button1);
                button1.Click += new EventHandler(button_Click);
            }
            if (buttonCount > 1)
            {
                SetButton(button2);
                button2.Click += new EventHandler(button_Click);
            }
            if (buttonCount > 2)
            {
                SetButton(button3);
                button3.Click += new EventHandler(button_Click);
            }

            Image icon = this.GetImage(iconFileName);
            if (icon != null)
            {
                this.pictureBox1.Image = icon;
                this.pictureBox1.Size = icon.Size;
                this.pictureBox1.Location = new Point(12, 12);
            }
            else
            {
                this.pictureBox1.Size = new Size(0, 0);
                this.pictureBox1.Location = new Point(0, 0);
            }

            this.Text = title;
            this.label1.Text = message;

            this.label1.Location = new Point(this.pictureBox1.Location.X + this.pictureBox1.Width + 12, 12);

            int labelBottom = label1.Height + label1.Location.Y;
            int iconBottom = this.pictureBox1.Height + this.pictureBox1.Location.Y;

            this.Width = label1.Width + label1.Location.X + 12;
            int bodyBottom = (labelBottom > iconBottom) ? labelBottom : iconBottom;

            if (iconBottom > labelBottom)
            {
                this.label1.Location = new Point(
                    this.pictureBox1.Location.X + this.pictureBox1.Width + 12,
                    12 + (this.pictureBox1.Height / 2) - (this.label1.Height / 2)
                );
            }

            int buttonLocationY = bodyBottom + 12;

            switch (buttonCount)
            {
                case 1:

                    this.AdjustWidth(button1.Width + 24);
                    button1.Location = new Point(this.Width / 2 - (button1.Width / 2), buttonLocationY);

                    break;

                case 2:

                    this.AdjustWidth(button1.Width + button2.Width + 34);

                    button1.Location = new Point(this.Width / 2 - ((button1.Width + button2.Width + 6) / 2), buttonLocationY);
                    button2.Location = new Point(this.Width / 2 - ((button1.Width + button2.Width + 6) / 2) + 6 + button1.Width, buttonLocationY);

                    break;

                case 3:

                    this.AdjustWidth(button1.Width + button2.Width + button3.Width + 40);

                    button1.Location = new Point(this.Width / 2 - ((button1.Width + button2.Width + button3.Width + 12) / 2), buttonLocationY);
                    button2.Location = new Point(this.Width / 2 - ((button1.Width + button2.Width + button3.Width + 12) / 2) + 6 + button1.Width, buttonLocationY);
                    button3.Location = new Point(this.Width / 2 - ((button1.Width + button2.Width + button3.Width + 12) / 2) + 12 + button1.Width + button2.Width, buttonLocationY);

                    break;
            }

            int buttonMarginY = (buttonCount == 0) ? 12 : 24 + (new Button()).Height;

            this.Height = 25 + bodyBottom + buttonMarginY;

            return this.ShowDialog();
        }

        void button_Click(object sender, EventArgs e)
        {
            this.DialogResult = (sender as Button).DialogResult;
        }

        private Image GetImage(string fileName)
        {
            Image image = null;

            try
            {
                if (fileName != "")
                {
                    image = Image.FromFile(fileName);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("摜݂܂ł");
            }

            return image;
        }

        private void AdjustWidth(int width)
        {
            if (this.Width < width)
            {
                this.Width = width;
            }
        }

        private Button SetButton(Button button)
        {
            Encoding sjis = Encoding.GetEncoding("shift_jis");
            button.Width = sjis.GetByteCount(button.Text) * 6;
            button.Width = (button.Width < 75) ? 75 : button.Width;
            this.Controls.Add(button);

            return button;
        }
    }
}